@@ -5,29 +5,19 @@ from typing import Any
 import ipywidgets
 import traitlets.traitlets as t
 
+from ._file_contents import FileContents
 from ._util import (
+    _ANYWIDGET_ID_KEY,
+    _CSS_KEY,
+    _DEFAULT_ESM,
+    _ESM_KEY,
     enable_custom_widget_manager_once,
     get_repr_metadata,
     in_colab,
+    try_file_contents,
 )
 from ._version import __version__
 
-_ANYWIDGET_ID_KEY = "_anywidget_id"
-_ESM_KEY = "_esm"
-_CSS_KEY = "_css"
-DEFAULT_ESM = """
-export function render(view) {
-  console.log("Dev note: No _esm defined for this widget:", view);
-  let url = "https://anywidget.dev/en/getting-started/";
-  view.el.innerHTML = `<p>
-    <strong>Dev note</strong>:
-    <a href='${url}' target='blank'>Implement an <code>_esm</code> attribute</a>
-    on AnyWidget subclass <code>${view.model.get('_anywidget_id')}</code>
-    to customize this widget.
-  </p>`;
-}
-"""
-
 
 class AnyWidget(ipywidgets.DOMWidget):  # type: ignore [misc]
     """Main AnyWidget base class."""
@@ -44,15 +34,21 @@ class AnyWidget(ipywidgets.DOMWidget):  # type: ignore [misc]
         super().__init__(*args, **kwargs)
 
         # Add anywidget JS/CSS source as traits if not registered
-        anywidget_traits = {
-            k: t.Unicode(getattr(self, k)).tag(sync=True)
-            for k in (_ESM_KEY, _CSS_KEY)
-            if hasattr(self, k) and not self.has_trait(k)
-        }
+        anywidget_traits = {}
+
+        for key in (_ESM_KEY, _CSS_KEY):
+            if hasattr(self, key) and not self.has_trait(key):
+                value = getattr(self, key)
+                anywidget_traits[key] = t.Unicode(str(value)).tag(sync=True)
+
+                if isinstance(value, FileContents):
+                    value.changed.connect(
+                        lambda new_contents, key=key: setattr(self, key, new_contents)
+                    )
 
         # show default _esm if not defined
         if not hasattr(self, _ESM_KEY):
-            anywidget_traits[_ESM_KEY] = t.Unicode(DEFAULT_ESM).tag(sync=True)
+            anywidget_traits[_ESM_KEY] = t.Unicode(_DEFAULT_ESM).tag(sync=True)
 
         # TODO: a better way to uniquely identify this subclasses?
         # We use the fully-qualified name to get an id which we
@@ -66,6 +62,15 @@ class AnyWidget(ipywidgets.DOMWidget):  # type: ignore [misc]
         if in_colab():
             enable_custom_widget_manager_once()
 
+    def __init_subclass__(cls, **kwargs: dict) -> None:
+        """Coerces _esm and _css to FileContents if they are files."""
+        super().__init_subclass__(**kwargs)
+        for key in (_ESM_KEY, _CSS_KEY) & cls.__dict__.keys():
+            # TODO: Upgrate to := when we drop Python 3.7
+            file_contents = try_file_contents(getattr(cls, key))
+            if file_contents:
+                setattr(cls, key, file_contents)
+
     if hasattr(ipywidgets.DOMWidget, "_repr_mimebundle_"):
         # ipywidgets v8
         def _repr_mimebundle_(self, **kwargs: dict) -> tuple[None | dict, dict]:
