@@ -1,11 +1,14 @@
+import pathlib
 import sys
 from unittest.mock import MagicMock
 
 import pytest
+from anywidget._file_contents import FileContents
 from anywidget._util import (
     get_repr_metadata,
     put_buffers,
     remove_buffers,
+    try_file_contents,
 )
 
 
@@ -95,3 +98,25 @@ def test_get_metadata(monkeypatch: pytest.MonkeyPatch):
             "colab": {"custom_widget_manager": {"url": "foo"}}
         }
     }
+
+
+def test_try_file_contents(tmp_path: pathlib.Path):
+    foo = tmp_path / "foo.txt"
+
+    assert try_file_contents(foo) is None
+
+    foo.write_text("foo")
+
+    file_contents = try_file_contents(foo)
+    assert isinstance(file_contents, FileContents)
+    assert file_contents._background_thread is not None
+    file_contents.stop_thread()  # stop the background thread for CI
+
+    site_packages = tmp_path / "site-packages"
+    site_packages.mkdir()
+    bar = site_packages / "bar.txt"
+    bar.write_text("bar")
+
+    file_contents = try_file_contents(bar)
+    assert isinstance(file_contents, FileContents)
+    assert file_contents._background_thread is None
